@@ -54,11 +54,19 @@ enum class ArrayDirectoryMode {
   VACUUM_FRAGMENTS  // Used when opening the array for fragment vacuuming.
 };
 
+// Forward declaration
+class WhiteboxArrayDirectory;
+
 /**
  * Manages the various URIs inside an array directory, considering
  * various versions of the on-disk TileDB format.
  */
 class ArrayDirectory {
+  /**
+   * Friends with its whitebox testing class.
+   */
+  friend class WhiteboxArrayDirectory;
+
  public:
   /* ********************************* */
   /*     CONSTRUCTORS & DESTRUCTORS    */
@@ -81,6 +89,8 @@ class ArrayDirectory {
    *     were created within timestamp range
    *    [`timestamp_start`, `timestamp_end`] will be considered when
    *     fetching URIs.
+   * @param consolidation_with_timestamps_config Ture if consolidation with
+   * timestamps is enabled in config, false if not enabled or not relevant
    * @param mode The mode to load the array directory in.
    */
   ArrayDirectory(
@@ -89,6 +99,7 @@ class ArrayDirectory {
       const URI& uri,
       uint64_t timestamp_start,
       uint64_t timestamp_end,
+      bool consolidation_with_timestamps_config,
       ArrayDirectoryMode mode = ArrayDirectoryMode::READ);
 
   /** Destructor. */
@@ -251,6 +262,9 @@ class ArrayDirectory {
   /** True if `load` has been run. */
   bool loaded_;
 
+  /** True if consolidation with timestamps is enabled in config */
+  bool consolidation_with_timestamps_config_;
+
   /* ********************************* */
   /*          PRIVATE METHODS          */
   /* ********************************* */
@@ -370,6 +384,20 @@ class ArrayDirectory {
   Status compute_array_schema_uris(
       const std::vector<URI>& array_schema_dir_uris);
 
+  /**
+   * Checks if a fragment overlaps with the array directory timestamp
+   * range. Overlap is partial or full depending on the consolidation
+   * type (with timestamps or not).
+   *
+   * @param fragment_timestamp_range Timestamp range of a given fragment.
+   * @param consolidation_with_timestamps True if consolidation includes
+   * timestamps, false otherwise.
+   * @return True if there is overlap, false otherwise.
+   */
+  bool timestamps_overlap(
+      const std::pair<uint64_t, uint64_t> fragment_timestamp_range,
+      const bool consolidation_with_timestamps) const;
+
   /** Returns true if the input URI is a vacuum file. */
   bool is_vacuum_file(const URI& uri) const;
 
@@ -394,6 +422,14 @@ class ArrayDirectory {
       const std::unordered_set<std::string>& ok_uris_set,
       const std::unordered_set<std::string>& consolidated_uris_set,
       int32_t* is_fragment) const;
+
+  /**
+   * Checks if consolidation with timestamps is supported for a fragment
+   *
+   * @param uri The fragment URI to be checked.
+   * @return True if supported, false otherwise
+   */
+  bool consolidation_with_timestamps_supported(const URI& uri) const;
 };
 
 }  // namespace sm
