@@ -131,8 +131,8 @@ ArraySchema::ArraySchema(
 
   // Create dimension map
   dim_map_.clear();
-  for (unsigned d = 0; d < domain_->dim_num(); ++d) {
-    auto dim = domain_->dimension(d);
+  for (dimension_size_type d = 0; d < domain_->dim_num(); ++d) {
+    auto dim{domain_->dimension_ptr(d)};
     dim_map_[dim->name()] = dim;
   }
 
@@ -204,7 +204,7 @@ const URI& ArraySchema::array_uri() const {
   return array_uri_;
 }
 
-const Attribute* ArraySchema::attribute(unsigned int id) const {
+const Attribute* ArraySchema::attribute(attribute_size_type id) const {
   if (id < attributes_.size())
     return attributes_[id].get();
   return nullptr;
@@ -215,8 +215,8 @@ const Attribute* ArraySchema::attribute(const std::string& name) const {
   return it == attribute_map_.end() ? nullptr : it->second;
 }
 
-unsigned int ArraySchema::attribute_num() const {
-  return (unsigned)attributes_.size();
+ArraySchema::attribute_size_type ArraySchema::attribute_num() const {
+  return static_cast<attribute_size_type>(attributes_.size());
 }
 
 const std::vector<shared_ptr<const Attribute>>& ArraySchema::attributes()
@@ -237,10 +237,14 @@ uint64_t ArraySchema::cell_size(const std::string& name) const {
   if (name == constants::coords) {
     auto dim_num = domain_->dim_num();
     assert(dim_num > 0);
-    auto coord_size = domain_->dimension(0)->coord_size();
+    auto coord_size{domain_->dimension_ptr(0)->coord_size()};
     return dim_num * coord_size;
   }
 
+  if (name == constants::timestamps) {
+    return constants::timestamp_size;
+  }
+
   // Attribute
   auto attr_it = attribute_map_.find(name);
   if (attr_it != attribute_map_.end()) {
@@ -263,7 +267,7 @@ uint64_t ArraySchema::cell_size(const std::string& name) const {
 
 unsigned int ArraySchema::cell_val_num(const std::string& name) const {
   // Special zipped coordinates
-  if (name == constants::coords)
+  if (name == constants::coords || name == constants::timestamps)
     return 1;
 
   // Attribute
@@ -302,7 +306,7 @@ Status ArraySchema::check() const {
   }
 
   if (array_type_ == ArrayType::DENSE) {
-    auto type = domain_->dimension(0)->type();
+    auto type{domain_->dimension_ptr(0)->type()};
     if (datatype_is_real(type)) {
       return LOG_STATUS(
           Status_ArraySchemaError("Array schema check failed; Dense arrays "
@@ -340,7 +344,7 @@ Status ArraySchema::check_attributes(
 }
 
 const FilterPipeline& ArraySchema::filters(const std::string& name) const {
-  if (name == constants::coords)
+  if (name == constants::coords || name == constants::timestamps)
     return coords_filters();
 
   // Attribute
@@ -363,12 +367,11 @@ bool ArraySchema::dense() const {
   return array_type_ == ArrayType::DENSE;
 }
 
-shared_ptr<const Dimension> ArraySchema::dimension(unsigned int i) const {
-  return domain_->dimension(i);
+const Dimension* ArraySchema::dimension_ptr(dimension_size_type i) const {
+  return domain_->dimension_ptr(i);
 }
 
-shared_ptr<const Dimension> ArraySchema::dimension(
-    const std::string& name) const {
+const Dimension* ArraySchema::dimension_ptr(const std::string& name) const {
   auto it = dim_map_.find(name);
   return it == dim_map_.end() ? nullptr : it->second;
 }
@@ -378,7 +381,7 @@ std::vector<std::string> ArraySchema::dim_names() const {
   std::vector<std::string> ret;
   ret.reserve(dim_num);
   for (uint32_t d = 0; d < dim_num; ++d)
-    ret.emplace_back(domain_->dimension(d)->name());
+    ret.emplace_back(domain_->dimension_ptr(d)->name());
 
   return ret;
 }
@@ -388,12 +391,12 @@ std::vector<Datatype> ArraySchema::dim_types() const {
   std::vector<Datatype> ret;
   ret.reserve(dim_num);
   for (uint32_t d = 0; d < dim_num; ++d)
-    ret.emplace_back(domain_->dimension(d)->type());
+    ret.emplace_back(domain_->dimension_ptr(d)->type());
 
   return ret;
 }
 
-unsigned int ArraySchema::dim_num() const {
+ArraySchema::dimension_size_type ArraySchema::dim_num() const {
   return domain_->dim_num();
 }
 
@@ -449,11 +452,12 @@ bool ArraySchema::is_attr(const std::string& name) const {
 }
 
 bool ArraySchema::is_dim(const std::string& name) const {
-  return this->dimension(name) != nullptr;
+  return this->dimension_ptr(name) != nullptr;
 }
 
 bool ArraySchema::is_field(const std::string& name) const {
-  return is_attr(name) || is_dim(name) || name == constants::coords;
+  return is_attr(name) || is_dim(name) || name == constants::coords ||
+         name == constants::timestamps;
 }
 
 bool ArraySchema::is_nullable(const std::string& name) const {
@@ -529,7 +533,10 @@ Layout ArraySchema::tile_order() const {
 Datatype ArraySchema::type(const std::string& name) const {
   // Special zipped coordinates attribute
   if (name == constants::coords)
-    return domain_->dimension(0)->type();
+    return domain_->dimension_ptr(0)->type();
+
+  if (name == constants::timestamps)
+    return constants::timestamp_type;
 
   // Attribute
   auto attr_it = attribute_map_.find(name);
@@ -544,7 +551,7 @@ Datatype ArraySchema::type(const std::string& name) const {
 
 bool ArraySchema::var_size(const std::string& name) const {
   // Special case for zipped coordinates
-  if (name == constants::coords)
+  if (name == constants::coords || name == constants::timestamps)
     return false;
 
   // Attribute
@@ -651,9 +658,11 @@ ArraySchema ArraySchema::deserialize(ConstBuffer* buff, const URI& uri) {
   if (!st.ok())
     throw std::runtime_error(
         "[ArraySchema::deserialize] Failed to load tile order.");
-  if (!tile_order_is_valid(tile_order_loaded).ok())
-    throw StatusException(Status_ArraySchemaError(
-        "[ArraySchema::deserialize] Invalid tile order."));
+  try {
+    ensure_tile_order_is_valid(tile_order_loaded);
+  } catch (std::exception& e) {
+    std::throw_with_nested(std::runtime_error("[ArraySchema::deserialize] "));
+  }
   Layout tile_order = Layout(tile_order_loaded);
 
   // Load cell order
@@ -662,9 +671,11 @@ ArraySchema ArraySchema::deserialize(ConstBuffer* buff, const URI& uri) {
   if (!st.ok())
     throw std::runtime_error(
         "[ArraySchema::deserialize] Failed to load cell order.");
-  if (!cell_order_is_valid(cell_order_loaded).ok())
-    throw StatusException(Status_ArraySchemaError(
-        "[ArraySchema::deserialize] Invalid cell order."));
+  try {
+    ensure_cell_order_is_valid(cell_order_loaded);
+  } catch (std::exception& e) {
+    std::throw_with_nested(std::runtime_error("[ArraySchema::deserialize] "));
+  }
   Layout cell_order = Layout(cell_order_loaded);
 
   // Load capacity
@@ -748,7 +759,7 @@ ArraySchema ArraySchema::deserialize(ConstBuffer* buff, const URI& uri) {
   }
 
   if (array_type == ArrayType::DENSE) {
-    auto type = domain.value()->dimension(0)->type();
+    auto type{domain.value()->dimension_ptr(0)->type()};
     if (datatype_is_real(type)) {
       throw StatusException(
           Status_ArraySchemaError("Array schema check failed; Dense arrays "
@@ -855,7 +866,7 @@ Status ArraySchema::set_domain(shared_ptr<Domain> domain) {
           Status_ArraySchemaError("Cannot set domain; In dense arrays, all "
                                   "dimensions must have the same datatype"));
 
-    auto type = domain->dimension(0)->type();
+    auto type{domain->dimension_ptr(0)->type()};
     if (!datatype_is_integer(type) && !datatype_is_datetime(type) &&
         !datatype_is_time(type)) {
       return LOG_STATUS(Status_ArraySchemaError(
@@ -875,8 +886,8 @@ Status ArraySchema::set_domain(shared_ptr<Domain> domain) {
   // Create dimension map
   dim_map_.clear();
   auto dim_num = domain_->dim_num();
-  for (unsigned d = 0; d < dim_num; ++d) {
-    auto dim = dimension(d);
+  for (dimension_size_type d = 0; d < dim_num; ++d) {
+    auto dim{dimension_ptr(d)};
     dim_map_[dim->name()] = dim;
   }
 
@@ -938,8 +949,8 @@ bool ArraySchema::check_attribute_dimension_names() const {
   auto dim_num = this->dim_num();
   for (auto attr : attributes_)
     names.insert(attr->name());
-  for (unsigned int i = 0; i < dim_num; ++i)
-    names.insert(domain_->dimension(i)->name());
+  for (dimension_size_type i = 0; i < dim_num; ++i)
+    names.insert(domain_->dimension_ptr(i)->name());
   return (names.size() == attributes_.size() + dim_num);
 }
 
@@ -962,8 +973,8 @@ Status ArraySchema::check_double_delta_compressor(
   // Error if any real dimension inherits the coord filters with DOUBLE DELTA.
   // A dimension inherits the filters when it has no filters.
   auto dim_num = domain_->dim_num();
-  for (unsigned d = 0; d < dim_num; ++d) {
-    auto dim = domain_->dimension(d);
+  for (dimension_size_type d = 0; d < dim_num; ++d) {
+    auto dim{domain_->dimension_ptr(d)};
     const auto& dim_filters = dim->filters();
     auto dim_type = dim->type();
     if (datatype_is_real(dim_type) && dim_filters.empty())
@@ -987,8 +998,8 @@ Status ArraySchema::check_string_compressor(
   // Error if there are also other filters set for a string dimension together
   // with RLE or Dictionary-encoding
   auto dim_num = domain_->dim_num();
-  for (unsigned d = 0; d < dim_num; ++d) {
-    auto dim = domain_->dimension(d);
+  for (dimension_size_type d = 0; d < dim_num; ++d) {
+    auto dim{domain_->dimension_ptr(d)};
     const auto& dim_filters = dim->filters();
     // if it's a var-length string dimension and there is no specific filter
     // list already set for that dimension (then coords_filters_ will be used)
