@@ -58,6 +58,15 @@ class LZ4 {
   static Status compress(
       int level, ConstBuffer* input_buffer, Buffer* output_buffer);
 
+  /**
+   * Overloaded compression function with default compression level.
+   *
+   * @param input_buffer Input buffer to read from.
+   * @param output_buffer Output buffer to write to the compressed data.
+   * @return Status
+   */
+  static Status compress(ConstBuffer* input_buffer, Buffer* output_buffer);
+
   /**
    * Decompression function.
    *
@@ -70,11 +79,15 @@ class LZ4 {
 
   /** Returns the default compression level. */
   static int default_level() {
-    return -1;
+    return default_level_;
   }
 
   /** Returns the compression overhead for the given input. */
   static uint64_t overhead(uint64_t nbytes);
+
+ private:
+  /** The default filter compression level. */
+  static constexpr int default_level_ = -1;
 };
 
 }  // namespace sm
