@@ -77,7 +77,7 @@ const std::string Config::SM_SKIP_EST_SIZE_PARTITIONING = "false";
 const std::string Config::SM_MEMORY_BUDGET = "5368709120";       // 5GB
 const std::string Config::SM_MEMORY_BUDGET_VAR = "10737418240";  // 10GB;
 const std::string Config::SM_QUERY_DENSE_READER = "refactored";
-const std::string Config::SM_QUERY_SPARSE_GLOBAL_ORDER_READER = "legacy";
+const std::string Config::SM_QUERY_SPARSE_GLOBAL_ORDER_READER = "refactored";
 const std::string Config::SM_QUERY_SPARSE_UNORDERED_WITH_DUPS_READER =
     "refactored";
 const std::string Config::SM_MEM_MALLOC_TRIM = "true";
@@ -111,6 +111,7 @@ const std::string Config::SM_CONSOLIDATION_MODE = "fragments";
 const std::string Config::SM_CONSOLIDATION_TIMESTAMP_START = "0";
 const std::string Config::SM_CONSOLIDATION_TIMESTAMP_END =
     std::to_string(UINT64_MAX);
+const std::string Config::SM_CONSOLIDATION_WITH_TIMESTAMPS = "false";
 const std::string Config::SM_VACUUM_MODE = "fragments";
 const std::string Config::SM_VACUUM_TIMESTAMP_START = "0";
 const std::string Config::SM_VACUUM_TIMESTAMP_END = std::to_string(UINT64_MAX);
@@ -280,9 +281,9 @@ Config::Config() {
       SM_CONSOLIDATION_TIMESTAMP_START;
   param_values_["sm.consolidation.timestamp_end"] =
       SM_CONSOLIDATION_TIMESTAMP_END;
+  param_values_["sm.consolidation.with_timestamps"] =
+      SM_CONSOLIDATION_WITH_TIMESTAMPS;
   param_values_["sm.vacuum.mode"] = SM_VACUUM_MODE;
-  param_values_["sm.vacuum.timestamp_start"] = SM_VACUUM_TIMESTAMP_START;
-  param_values_["sm.vacuum.timestamp_end"] = SM_VACUUM_TIMESTAMP_END;
   param_values_["sm.var_offsets.bitsize"] = SM_OFFSETS_BITSIZE;
   param_values_["sm.var_offsets.extra_element"] = SM_OFFSETS_EXTRA_ELEMENT;
   param_values_["sm.var_offsets.mode"] = SM_OFFSETS_FORMAT_MODE;
@@ -619,12 +620,11 @@ Status Config::unset(const std::string& param) {
   } else if (param == "sm.consolidation.timestamp_end") {
     param_values_["sm.consolidation.timestamp_end"] =
         SM_CONSOLIDATION_TIMESTAMP_END;
+  } else if (param == "sm.consolidation.with_timestamps") {
+    param_values_["sm.consolidation.with_timestamps"] =
+        SM_CONSOLIDATION_WITH_TIMESTAMPS;
   } else if (param == "sm.vacuum.mode") {
     param_values_["sm.vacuum.mode"] = SM_VACUUM_MODE;
-  } else if (param == "sm.vacuum.timestamp_start") {
-    param_values_["sm.vacuum.timestamp_start"] = SM_VACUUM_TIMESTAMP_START;
-  } else if (param == "sm.vacuum.timestamp_end") {
-    param_values_["sm.vacuum.timestamp_end"] = SM_VACUUM_TIMESTAMP_END;
   } else if (param == "sm.var_offsets.bitsize") {
     param_values_["sm.var_offsets.bitsize"] = SM_OFFSETS_BITSIZE;
   } else if (param == "sm.var_offsets.extra_element") {
