@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2021 TileDB, Inc.
+ * @copyright Copyright (c) 2021-2022 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -38,6 +38,7 @@
 
 #include "tiledb/common/status.h"
 #include "tiledb/sm/misc/constants.h"
+#include "tiledb/stdx/utility/to_underlying.h"
 
 using namespace tiledb::common;
 
@@ -94,6 +95,24 @@ inline Status query_condition_op_enum(
   return Status::Ok();
 }
 
+inline void ensure_qc_op_is_valid(QueryConditionOp query_condition_op) {
+  auto qc_op_enum{::stdx::to_underlying(query_condition_op)};
+  if (qc_op_enum > 5) {
+    throw std::runtime_error(
+        "Invalid Query Condition Op " + std::to_string(qc_op_enum));
+  }
+}
+
+inline void ensure_qc_op_string_is_valid(const std::string& qc_op_str) {
+  QueryConditionOp qc_op = QueryConditionOp::LT;
+  Status st{query_condition_op_enum(qc_op_str, &qc_op)};
+  if (!st.ok()) {
+    throw std::runtime_error(
+        "Invalid Query Condition Op string \"" + qc_op_str + "\"");
+  }
+  ensure_qc_op_is_valid(qc_op);
+}
+
 }  // namespace sm
 }  // namespace tiledb
 
