@@ -75,7 +75,7 @@ GlobalOrderWriter::GlobalOrderWriter(
     Subarray& subarray,
     Layout layout,
     std::vector<WrittenFragmentInfo>& written_fragment_info,
-    bool disable_check_global_order,
+    bool disable_checks_consolidation,
     Query::CoordsInfo& coords_info,
     URI fragment_uri)
     : WriterBase(
@@ -88,7 +88,7 @@ GlobalOrderWriter::GlobalOrderWriter(
           subarray,
           layout,
           written_fragment_info,
-          disable_check_global_order,
+          disable_checks_consolidation,
           coords_info,
           fragment_uri) {
 }
@@ -157,7 +157,7 @@ Status GlobalOrderWriter::check_coord_dups() const {
   std::vector<const unsigned char*> buffs_var(dim_num);
   std::vector<uint64_t*> buffs_var_sizes(dim_num);
   for (unsigned d = 0; d < dim_num; ++d) {
-    const auto& dim_name = array_schema_.dimension(d)->name();
+    const auto& dim_name{array_schema_.dimension_ptr(d)->name()};
     buffs[d] = (const unsigned char*)buffers_.find(dim_name)->second.buffer_;
     coord_sizes[d] = array_schema_.cell_size(dim_name);
     buffs_var[d] =
@@ -173,7 +173,7 @@ Status GlobalOrderWriter::check_coord_dups() const {
         // Check for duplicate in adjacent cells
         bool found_dup = true;
         for (unsigned d = 0; d < dim_num; ++d) {
-          auto dim = array_schema_.dimension(d);
+          auto dim{array_schema_.dimension_ptr(d)};
           if (!dim->var_size()) {  // Fixed-sized dimensions
             if (memcmp(
                     buffs[d] + i * coord_sizes[d],
@@ -402,7 +402,7 @@ Status GlobalOrderWriter::compute_coord_dups(
   std::vector<const unsigned char*> buffs_var(dim_num);
   std::vector<uint64_t*> buffs_var_sizes(dim_num);
   for (unsigned d = 0; d < dim_num; ++d) {
-    const auto& dim_name = array_schema_.dimension(d)->name();
+    const auto& dim_name{array_schema_.dimension_ptr(d)->name()};
     buffs[d] = (const unsigned char*)buffers_.find(dim_name)->second.buffer_;
     coord_sizes[d] = array_schema_.cell_size(dim_name);
     buffs_var[d] =
@@ -419,7 +419,7 @@ Status GlobalOrderWriter::compute_coord_dups(
         // Check for duplicate in adjacent cells
         bool found_dup = true;
         for (unsigned d = 0; d < dim_num; ++d) {
-          auto dim = array_schema_.dimension(d);
+          auto dim{array_schema_.dimension_ptr(d)};
           if (!dim->var_size()) {  // Fixed-sized dimensions
             if (memcmp(
                     buffs[d] + i * coord_sizes[d],
@@ -768,7 +768,11 @@ Status GlobalOrderWriter::prepare_full_tiles_fixed(
 
   // First fill the last tile
   auto& last_tile = global_write_state_->last_tiles_[name][0];
-  auto& last_tile_validity = global_write_state_->last_tiles_[name][1];
+  decltype(&global_write_state_->last_tiles_[name][1]) last_tile_validity =
+      nullptr;
+  if (nullable) {
+    last_tile_validity = &global_write_state_->last_tiles_[name][1];
+  }
   uint64_t cell_idx = 0;
   uint64_t last_tile_cell_idx =
       global_write_state_->cells_written_[name] % cell_num_per_tile;
@@ -780,7 +784,7 @@ Status GlobalOrderWriter::prepare_full_tiles_fixed(
             last_tile_cell_idx * cell_size,
             cell_size));
         if (nullable) {
-          RETURN_NOT_OK(last_tile_validity.write(
+          RETURN_NOT_OK(last_tile_validity->write(
               buffer_validity + cell_idx * constants::cell_validity_size,
               last_tile_cell_idx * constants::cell_validity_size,
               constants::cell_validity_size));
@@ -796,7 +800,7 @@ Status GlobalOrderWriter::prepare_full_tiles_fixed(
               last_tile_cell_idx * cell_size,
               cell_size));
           if (nullable) {
-            RETURN_NOT_OK(last_tile_validity.write(
+            RETURN_NOT_OK(last_tile_validity->write(
                 buffer_validity + cell_idx * constants::cell_validity_size,
                 last_tile_cell_idx * constants::cell_validity_size,
                 constants::cell_validity_size));
@@ -831,7 +835,7 @@ Status GlobalOrderWriter::prepare_full_tiles_fixed(
     if (last_tile_cell_idx == cell_num_per_tile) {
       (*tiles)[0].swap(last_tile);
       if (nullable) {
-        (*tiles)[1].swap(last_tile_validity);
+        (*tiles)[1].swap(*last_tile_validity);
       }
       tile_idx += t;
     } else if (last_tile_cell_idx != 0) {
@@ -889,7 +893,7 @@ Status GlobalOrderWriter::prepare_full_tiles_fixed(
           last_tile_cell_idx * cell_size,
           cell_size));
       if (nullable) {
-        RETURN_NOT_OK(last_tile_validity.write(
+        RETURN_NOT_OK(last_tile_validity->write(
             buffer_validity + cell_idx * constants::cell_validity_size,
             last_tile_cell_idx * constants::cell_validity_size,
             constants::cell_validity_size));
@@ -903,7 +907,7 @@ Status GlobalOrderWriter::prepare_full_tiles_fixed(
             last_tile_cell_idx * cell_size,
             cell_size));
         if (nullable) {
-          RETURN_NOT_OK(last_tile_validity.write(
+          RETURN_NOT_OK(last_tile_validity->write(
               buffer_validity + cell_idx * constants::cell_validity_size,
               last_tile_cell_idx * constants::cell_validity_size,
               constants::cell_validity_size));
@@ -945,7 +949,10 @@ Status GlobalOrderWriter::prepare_full_tiles_var(
   auto& last_tile_vector = global_write_state_->last_tiles_[name];
   auto& last_tile = last_tile_vector[0];
   auto& last_tile_var = last_tile_vector[1];
-  auto& last_tile_validity = last_tile_vector[2];
+  decltype(&last_tile_vector[2]) last_tile_validity = nullptr;
+  if (nullable) {
+    last_tile_validity = &last_tile_vector[2];
+  }
   auto& last_var_offset = global_write_state_->last_var_offsets_[name];
   uint64_t cell_idx = 0;
   uint64_t last_tile_cell_idx =
@@ -973,7 +980,7 @@ Status GlobalOrderWriter::prepare_full_tiles_var(
 
         // Write validity value(s).
         if (nullable)
-          RETURN_NOT_OK(last_tile_validity.write(
+          RETURN_NOT_OK(last_tile_validity->write(
               buffer_validity + cell_idx,
               last_tile_cell_idx * constants::cell_validity_size,
               constants::cell_validity_size));
@@ -1005,7 +1012,7 @@ Status GlobalOrderWriter::prepare_full_tiles_var(
 
           // Write validity value(s).
           if (nullable)
-            RETURN_NOT_OK(last_tile_validity.write(
+            RETURN_NOT_OK(last_tile_validity->write(
                 buffer_validity + cell_idx,
                 last_tile_cell_idx * constants::cell_validity_size,
                 constants::cell_validity_size));
@@ -1044,7 +1051,7 @@ Status GlobalOrderWriter::prepare_full_tiles_var(
       (*tiles)[0].swap(last_tile);
       (*tiles)[1].swap(last_tile_var);
       if (nullable) {
-        (*tiles)[2].swap(last_tile_validity);
+        (*tiles)[2].swap(*last_tile_validity);
       }
       tile_idx += t;
     } else if (last_tile_cell_idx != 0) {
@@ -1161,7 +1168,7 @@ Status GlobalOrderWriter::prepare_full_tiles_var(
 
       // Write validity value(s).
       if (nullable)
-        RETURN_NOT_OK(last_tile_validity.write(
+        RETURN_NOT_OK(last_tile_validity->write(
             buffer_validity + cell_idx,
             last_tile_cell_idx * constants::cell_validity_size,
             constants::cell_validity_size));
@@ -1189,7 +1196,7 @@ Status GlobalOrderWriter::prepare_full_tiles_var(
 
         // Write validity value(s).
         if (nullable)
-          RETURN_NOT_OK(last_tile_validity.write(
+          RETURN_NOT_OK(last_tile_validity->write(
               buffer_validity + cell_idx,
               last_tile_cell_idx * constants::cell_validity_size,
               constants::cell_validity_size));
