@@ -39,6 +39,7 @@
 #include "tiledb/common/status.h"
 #include "tiledb/sm/array_schema/dimension.h"
 #include "tiledb/sm/array_schema/tile_domain.h"
+#include "tiledb/sm/fragment/fragment_metadata.h"
 #include "tiledb/sm/misc/types.h"
 #include "tiledb/sm/query/query_condition.h"
 #include "tiledb/sm/query/result_cell_slab.h"
@@ -163,6 +164,24 @@ class ReaderBase : public StrategyBase {
   /** The fragment metadata that the reader will focus on. */
   std::vector<shared_ptr<FragmentMetadata>> fragment_metadata_;
 
+  /** Disable the tile cache or not. */
+  bool disable_cache_;
+
+  /**
+   * The condition to apply on results when there is partial time overlap
+   * with at least one fragment
+   * */
+  QueryCondition partial_overlap_condition_;
+
+  /** If the user requested timestamps attribute in the query */
+  bool user_requested_timestamps_;
+
+  /**
+   * If the special timestamps attribute should be loaded to memory for
+   * this query
+   */
+  bool use_timestamps_;
+
   /* ********************************* */
   /*         PROTECTED METHODS         */
   /* ********************************* */
@@ -196,6 +215,34 @@ class ReaderBase : public StrategyBase {
   /** Correctness checks validity buffer sizes in `buffers_`. */
   Status check_validity_buffer_sizes() const;
 
+  /**
+   * Skip read/unfilter operations for timestamps attribute and fragments
+   * without timestamps.
+   */
+  inline bool timestamps_not_present(
+      const std::string& name, const unsigned f) const {
+    return name == constants::timestamps && !include_timestamps(f);
+  }
+
+  /**
+   * Checks if timestamps should be loaded for a fragment.
+   *
+   * @param f Fragment index.
+   * @return True if timestamps should be included, false if they are not.
+   * needed.
+   */
+  bool include_timestamps(const unsigned f) const;
+
+  /**
+   * Returns the fragment timestamp for a result tile.
+   *
+   * @param rt Result tile.
+   * @return fragment timestamp.
+   */
+  inline uint64_t fragment_timestamp(ResultTile* rt) const {
+    return fragment_metadata_[rt->frag_idx()]->timestamp_range().first;
+  }
+
   /**
    * Loads tile offsets for each attribute/dimension name into
    * their associated element in `fragment_metadata_`.
@@ -207,6 +254,21 @@ class ReaderBase : public StrategyBase {
   Status load_tile_offsets(
       Subarray& subarray, const std::vector<std::string>& names);
 
+  /**
+   * Checks if at least one fragment overlaps partially with the
+   * time at which the read is taking place.
+   *
+   * @return True if at least one fragment partially overlaps.
+   */
+  bool partial_consolidated_fragment_overlap() const;
+
+  /**
+   * Add a condition for partial time overlap based on array open and
+   * end times, to be used to filter out results on fragments that have
+   * been consolidated with timestamps
+   */
+  Status add_partial_overlap_condition();
+
   /**
    * Loads tile var sizes for each attribute/dimension name into
    * their associated element in `fragment_metadata_`.
@@ -286,13 +348,11 @@ class ReaderBase : public StrategyBase {
    * @param names The attribute names.
    * @param result_tiles The retrieved tiles will be stored inside the
    *     `ResultTile` instances in this vector.
-   * @param disable_cache Disable the tile cache or not.
    * @return Status
    */
   Status read_attribute_tiles(
       const std::vector<std::string>& names,
-      const std::vector<ResultTile*>& result_tiles,
-      const bool disable_cache = false) const;
+      const std::vector<ResultTile*>& result_tiles) const;
 
   /**
    * Concurrently executes across each name in `names` and each result tile
@@ -304,13 +364,11 @@ class ReaderBase : public StrategyBase {
    * @param names The coordinate/dimension names.
    * @param result_tiles The retrieved tiles will be stored inside the
    *     `ResultTile` instances in this vector.
-   * @param disable_cache Disable the tile cache or not.
    * @return Status
    */
   Status read_coordinate_tiles(
       const std::vector<std::string>& names,
-      const std::vector<ResultTile*>& result_tiles,
-      const bool disable_cache = false) const;
+      const std::vector<ResultTile*>& result_tiles) const;
 
   /**
    * Retrieves the tiles on a list of attribute or dimension and stores it
@@ -322,13 +380,11 @@ class ReaderBase : public StrategyBase {
    * @param names The attribute/dimension names.
    * @param result_tiles The retrieved tiles will be stored inside the
    *     `ResultTile` instances in this vector.
-   * @param disable_cache Disable the tile cache or not.
    * @return Status
    */
   Status read_tiles(
       const std::vector<std::string>& names,
-      const std::vector<ResultTile*>& result_tiles,
-      const bool disable_cache = false) const;
+      const std::vector<ResultTile*>& result_tiles) const;
 
   /**
    * Filters the tiles on a particular attribute/dimension from all input
@@ -445,13 +501,11 @@ class ReaderBase : public StrategyBase {
    *
    * @param name Attribute/dimension whose tiles will be unfiltered.
    * @param result_tiles Vector containing the tiles to be unfiltered.
-   * @param disable_cache Disable the filtered buffers cache or not.
    * @return Status
    */
   Status unfilter_tiles(
       const std::string& name,
-      const std::vector<ResultTile*>& result_tiles,
-      const bool disable_cache = false) const;
+      const std::vector<ResultTile*>& result_tiles) const;
 
   /**
    * Runs the input fixed-sized tile for the input attribute or dimension
