@@ -89,6 +89,7 @@ void ResultTile::swap(ResultTile& tile) {
   std::swap(frag_idx_, tile.frag_idx_);
   std::swap(tile_idx_, tile.tile_idx_);
   std::swap(attr_tiles_, tile.attr_tiles_);
+  std::swap(timestamps_tile_, tile.timestamps_tile_);
   std::swap(coords_tile_, tile.coords_tile_);
   std::swap(coord_tiles_, tile.coord_tiles_);
   std::swap(compute_results_dense_func_, tile.compute_results_dense_func_);
@@ -154,8 +155,14 @@ void ResultTile::erase_tile(const std::string& name) {
 
 void ResultTile::init_attr_tile(const std::string& name) {
   // Nothing to do for the special zipped coordinates tile
-  if (name == constants::coords)
+  if (name == constants::coords) {
     return;
+  }
+
+  if (name == constants::timestamps) {
+    timestamps_tile_ = TileTuple(Tile(), Tile(), Tile());
+    return;
+  }
 
   // Handle attributes
   for (auto& at : attr_tiles_) {
@@ -177,8 +184,14 @@ void ResultTile::init_coord_tile(const std::string& name, unsigned dim_idx) {
 
 ResultTile::TileTuple* ResultTile::tile_tuple(const std::string& name) {
   // Handle zipped coordinates tile
-  if (name == constants::coords)
+  if (name == constants::coords) {
     return &coords_tile_;
+  }
+
+  // Handle timestamps tile
+  if (name == constants::timestamps) {
+    return &*timestamps_tile_;
+  }
 
   // Handle attribute tile
   for (auto& at : attr_tiles_) {
@@ -254,7 +267,7 @@ bool ResultTile::same_coords(
     const ResultTile& rt, uint64_t pos_a, uint64_t pos_b) const {
   auto dim_num = coord_tiles_.size();
   for (unsigned d = 0; d < dim_num; ++d) {
-    if (!domain_->dimension(d)->var_size()) {  // Fixed-sized
+    if (!domain_->dimension_ptr(d)->var_size()) {  // Fixed-sized
       if (std::memcmp(coord(pos_a, d), rt.coord(pos_b, d), coord_size(d)) != 0)
         return false;
     } else {  // Var-sized
@@ -266,6 +279,15 @@ bool ResultTile::same_coords(
   return true;
 }
 
+bool ResultTile::same_coords(uint64_t pos_a, uint64_t pos_b) const {
+  return same_coords(*this, pos_a, pos_b);
+}
+
+uint64_t ResultTile::timestamp(uint64_t pos) {
+  const auto& tile = std::get<0>(*this->tile_tuple(constants::timestamps));
+  return tile.data_as<uint64_t>()[pos];
+}
+
 unsigned ResultTile::frag_idx() const {
   return frag_idx_;
 }
@@ -322,7 +344,7 @@ Status ResultTile::read(
     assert(name != constants::coords);
     int dim_offset = 0;
     for (uint32_t i = 0; i < domain_->dim_num(); ++i) {
-      if (domain_->dimension(i)->name() == name) {
+      if (domain_->dimension_ptr(i)->name() == name) {
         dim_offset = i;
         break;
       }
@@ -704,6 +726,78 @@ void ResultTile::compute_results_sparse(
   }
 }
 
+template <class BitmapType>
+void ResultTile::compute_results_count_sparse_string_range(
+    const std::vector<std::pair<std::string_view, std::string_view>>
+        cached_ranges,
+    const char* buff_str,
+    const uint64_t* buff_off,
+    const uint64_t cell_num,
+    const uint64_t buff_str_size,
+    const uint64_t start,
+    const uint64_t end,
+    std::vector<BitmapType>& result_count) {
+  const bool non_overlapping = std::is_same<BitmapType, uint8_t>::value;
+
+  // Process all cells.
+  for (uint64_t pos = start; pos <= end; ++pos) {
+    if (result_count[pos] == 0)
+      continue;
+
+    uint64_t c_offset = buff_off[pos];
+    uint64_t c_size = (pos < cell_num - 1) ? buff_off[pos + 1] - c_offset :
+                                             buff_str_size - c_offset;
+
+    std::string_view str(buff_str + c_offset, c_size);
+
+    // Binary search to find the first range containing the cell.
+    auto it = std::lower_bound(
+        cached_ranges.begin(),
+        cached_ranges.end(),
+        str,
+        [&](const std::pair<std::string_view, std::string_view>& cached_range,
+            const std::string_view& value) {
+          return cached_range.second < value;
+        });
+
+    // If we didn't find a range we can set count to 0 and skip to next.
+    if (it == cached_ranges.end()) {
+      result_count[pos] = 0;
+      continue;
+    }
+    uint64_t start_range_idx = std::distance(cached_ranges.begin(), it);
+
+    uint64_t end_range_idx = 0;
+    if constexpr (non_overlapping) {
+      end_range_idx = start_range_idx + 1;
+    } else {
+      // Binary search to find the last range containing the cell.
+      auto it2 = std::lower_bound(
+          it,
+          cached_ranges.end(),
+          str,
+          [&](const std::pair<std::string_view, std::string_view>& cached_range,
+              const std::string_view& value) {
+            return cached_range.first <= value;
+          });
+
+      end_range_idx = std::distance(it, it2) + start_range_idx;
+    }
+
+    // Iterate through all relevant ranges and compute the count for this
+    // dim.
+    uint64_t count = 0;
+    for (uint64_t j = start_range_idx; j < end_range_idx; ++j) {
+      auto& range = cached_ranges[j];
+      count += str_coord_intersects(
+          c_offset, c_size, buff_str, range.first, range.second);
+    }
+
+    // Multiply the past count by this dimension's count.
+    result_count[pos] *= count;
+  }
+}
+
 template <class BitmapType>
 void ResultTile::compute_results_count_sparse_string(
     const ResultTile* result_tile,
@@ -735,6 +829,14 @@ void ResultTile::compute_results_count_sparse_string(
   auto buff_str = static_cast<const char*>(coord_tile_str.data());
   auto buff_str_size = coord_tile_str.size();
 
+  // Cache start_str/end_str for all ranges.
+  std::vector<std::pair<std::string_view, std::string_view>> cached_ranges;
+  cached_ranges.reserve(range_indexes.size());
+
+  for (auto& i : range_indexes) {
+    cached_ranges.emplace_back(ranges[i].start_str(), ranges[i].end_str());
+  }
+
   // For row-major cell orders, the first dimension is sorted.
   // For col-major cell orders, the last dimension is sorted.
   // If the coordinate value at index 0 is equivalent to the
@@ -797,13 +899,12 @@ void ResultTile::compute_results_count_sparse_string(
           strncmp(first_c_coord, second_coord, first_c_size) == 0) {
         uint64_t count = 0;
         for (auto i : range_indexes) {
-          auto& range = ranges[i];
           count += str_coord_intersects(
               first_c_offset,
               first_c_size,
               buff_str,
-              range.start_str(),
-              range.end_str());
+              cached_ranges[i].first,
+              cached_ranges[i].second);
         }
 
         for (uint64_t pos = first_c_pos; pos < first_c_pos + c_partition_size;
@@ -812,20 +913,15 @@ void ResultTile::compute_results_count_sparse_string(
         }
       } else {
         // Compute results
-        uint64_t c_offset = 0, c_size = 0;
-        for (uint64_t pos = first_c_pos; pos <= last_c_pos; ++pos) {
-          c_offset = buff_off[pos];
-          c_size = (pos < cell_num - 1) ? buff_off[pos + 1] - c_offset :
-                                          buff_str_size - c_offset;
-          uint64_t count = 0;
-          for (auto i : range_indexes) {
-            auto& range = ranges[i];
-            count += str_coord_intersects(
-                c_offset, c_size, buff_str, range.start_str(), range.end_str());
-          }
-
-          result_count[pos] = count;
-        }
+        compute_results_count_sparse_string_range(
+            cached_ranges,
+            buff_str,
+            buff_off,
+            cell_num,
+            buff_str_size,
+            first_c_pos,
+            last_c_pos,
+            result_count);
       }
     }
 
@@ -873,23 +969,15 @@ void ResultTile::compute_results_count_sparse_string(
       // Here, we know that there is at least one `1` value within the
       // `r_count` values within this partition. We must check
       // each value for an intersection.
-      uint64_t c_offset = 0, c_size = 0;
-      for (uint64_t pos = i; pos < i + partition_size; ++pos) {
-        if (result_count[pos] == 0)
-          continue;
-
-        c_offset = buff_off[pos];
-        c_size = (pos < cell_num - 1) ? buff_off[pos + 1] - c_offset :
-                                        buff_str_size - c_offset;
-        uint64_t count = 0;
-        for (auto j : range_indexes) {
-          auto& range = ranges[j];
-          count += str_coord_intersects(
-              c_offset, c_size, buff_str, range.start_str(), range.end_str());
-        }
-
-        result_count[pos] *= count;
-      }
+      compute_results_count_sparse_string_range(
+          cached_ranges,
+          buff_str,
+          buff_off,
+          cell_num,
+          buff_str_size,
+          i,
+          i + partition_size - 1,
+          result_count);
     }
   }
 }
@@ -928,7 +1016,7 @@ void ResultTile::compute_results_count_sparse(
               range_indexes.end(),
               c,
               [&](const uint64_t& index, const T& value) {
-                return ((const T*)ranges[index].start())[1] < value;
+                return ((const T*)ranges[index].start_fixed())[1] < value;
               });
 
           // If we didn't find a range we can set count to 0 and skip to next.
@@ -944,21 +1032,17 @@ void ResultTile::compute_results_count_sparse(
               range_indexes.end(),
               c,
               [&](const uint64_t& index, const T& value) {
-                return ((const T*)ranges[index].start())[0] < value;
+                return ((const T*)ranges[index].start_fixed())[0] <= value;
               });
 
-          // If the upper bound isn't the end add +1 to the index.
-          uint64_t offset = 0;
-          if (it2 != range_indexes.end())
-            offset = 1;
-          uint64_t end_range_idx =
-              std::distance(it, it2) + start_range_idx + offset;
+          uint64_t end_range_idx = std::distance(it, it2) + start_range_idx;
 
           // Iterate through all relevant ranges and compute the count for this
           // dim.
           uint64_t count = 0;
           for (uint64_t j = start_range_idx; j < end_range_idx; ++j) {
-            const auto& range = (const T*)ranges[range_indexes[j]].start();
+            const auto& range =
+                (const T*)ranges[range_indexes[j]].start_fixed();
             count += c >= range[0] && c <= range[1];
           }
 
@@ -981,7 +1065,7 @@ void ResultTile::compute_results_count_sparse(
         T c = coords[pos * dim_num + dim_idx];
         uint64_t count = 0;
         for (uint64_t i = 0; i < range_indexes.size(); i++) {
-          const auto& range = (const T*)ranges[range_indexes[i]].start();
+          const auto& range = (const T*)ranges[range_indexes[i]].start_fixed();
           count += c >= range[0] && c <= range[1];
         }
 
@@ -1077,7 +1161,7 @@ void ResultTile::set_compute_results_func() {
   compute_results_count_sparse_uint8_t_func_.resize(dim_num);
   compute_results_count_sparse_uint64_t_func_.resize(dim_num);
   for (unsigned d = 0; d < dim_num; ++d) {
-    auto dim = domain_->dimension(d);
+    auto dim{domain_->dimension_ptr(d)};
     switch (dim->type()) {
       case Datatype::INT32:
         compute_results_dense_func_[d] = compute_results_dense<int32_t>;
