@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2022 TileDB, Inc.
  * @copyright Copyright (c) 2016 MIT and Intel Corporation
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -31,6 +31,8 @@
  * This file defines serialization for the Query class
  */
 
+#include <sstream>
+
 // clang-format off
 #ifdef TILEDB_SERIALIZATION
 #include <capnp/compat/json.h>
@@ -42,6 +44,7 @@
 #endif
 // clang-format on
 
+#include "tiledb/sm/query/ast/query_ast.h"
 #include "tiledb/sm/query/query.h"
 #include "tiledb/common/common.h"
 #include "tiledb/common/heap_memory.h"
@@ -140,7 +143,7 @@ Status subarray_to_capnp(
   const uint32_t dim_num = subarray->dim_num();
   auto ranges_builder = builder->initRanges(dim_num);
   for (uint32_t i = 0; i < dim_num; i++) {
-    const auto datatype = schema.dimension(i)->type();
+    const auto datatype{schema.dimension_ptr(i)->type()};
     auto range_builder = ranges_builder[i];
     const auto& ranges = subarray->ranges_for_dim(i);
     range_builder.setType(datatype_str(datatype));
@@ -609,43 +612,163 @@ Status dense_read_state_from_capnp(
   return Status::Ok();
 }
 
+/**
+ * @brief Validation function for the field name of an AST node.
+ *
+ * @param field_name Query Condition AST node field name.
+ */
+void ensure_qc_field_name_is_valid(const std::string& field_name) {
+  if (field_name == "") {
+    throw std::runtime_error(
+        "Invalid Query Condition field name " + field_name);
+  }
+}
+
+/**
+ * @brief Validation function for the condition value of an AST node.
+ *
+ * @param data Query Condition AST node condition value byte vector start.
+ * @param size The number of bytes in the byte vector.
+ */
+void ensure_qc_condition_value_is_valid(const void* data, size_t size) {
+  if (size != 0 && data == nullptr) {
+    // Getting the address of data in string form.
+    std::ostringstream address;
+    address << data;
+    std::string address_name = address.str();
+    throw std::runtime_error(
+        "Invalid Query Condition condition value " + address_name +
+        " with size " + std::to_string(size));
+  }
+}
+
+/**
+ * @brief Validation function for the capnp representation of the
+ * condition value of an AST node.
+ *
+ * @param vec The vector representing the Query Condition AST node
+ * condition vector.
+ */
+void ensure_qc_capnp_condition_value_is_valid(const kj::Vector<uint8_t>& vec) {
+  ensure_qc_condition_value_is_valid(vec.begin(), vec.size());
+}
+
+static Status condition_ast_to_capnp(
+    const tdb_unique_ptr<ASTNode>& node, capnp::ASTNode::Builder* ast_builder) {
+  if (!node->is_expr()) {
+    // Store the boolean expression tag.
+    ast_builder->setIsExpression(false);
+
+    // Validate and store the field name.
+    const std::string field_name = node->get_field_name();
+    ensure_qc_field_name_is_valid(field_name);
+    ast_builder->setFieldName(field_name);
+
+    // Copy the condition value into a capnp vector of bytes.
+    const UntypedDatumView& value = node->get_condition_value_view();
+    auto capnpValue = kj::Vector<uint8_t>();
+    capnpValue.addAll(kj::ArrayPtr<uint8_t>(
+        const_cast<uint8_t*>(reinterpret_cast<const uint8_t*>(value.content())),
+        value.size()));
+
+    // Validate and store the condition value vector of bytes.
+    ensure_qc_capnp_condition_value_is_valid(capnpValue);
+    ast_builder->setValue(capnpValue.asPtr());
+
+    // Validate and store the query condition op.
+    const std::string op_str = query_condition_op_str(node->get_op());
+    ensure_qc_op_string_is_valid(op_str);
+    ast_builder->setOp(op_str);
+  } else {
+    // Store the boolean expression tag.
+    ast_builder->setIsExpression(true);
+
+    // We assume that the serialized values of the child nodes are validated
+    // properly.
+    auto children_builder =
+        ast_builder->initChildren(node->get_children().size());
+
+    for (size_t i = 0; i < node->get_children().size(); ++i) {
+      auto child_builder = children_builder[i];
+      condition_ast_to_capnp(node->get_children()[i], &child_builder);
+    }
+
+    // Validate and store the query condition combination op.
+    const std::string op_str =
+        query_condition_combination_op_str(node->get_combination_op());
+    ensure_qc_combo_op_string_is_valid(op_str);
+    ast_builder->setCombinationOp(op_str);
+  }
+  return Status::Ok();
+}
+
+static void clause_to_capnp(
+    const tdb_unique_ptr<ASTNode>& node,
+    capnp::ConditionClause::Builder* clause_builder) {
+  // Validate and store the field name.
+  std::string field_name = node->get_field_name();
+  ensure_qc_field_name_is_valid(field_name);
+  clause_builder->setFieldName(field_name);
+
+  // Copy the condition value into a capnp vector of bytes.
+  const UntypedDatumView& value = node->get_condition_value_view();
+  auto capnpValue = kj::Vector<uint8_t>();
+  capnpValue.addAll(kj::ArrayPtr<uint8_t>(
+      const_cast<uint8_t*>(reinterpret_cast<const uint8_t*>(value.content())),
+      value.size()));
+
+  // Validate and store the condition value vector of bytes.
+  ensure_qc_capnp_condition_value_is_valid(capnpValue);
+  clause_builder->setValue(capnpValue.asPtr());
+
+  // Validate and store the query condition op.
+  const std::string op_str = query_condition_op_str(node->get_op());
+  ensure_qc_op_string_is_valid(op_str);
+  clause_builder->setOp(op_str);
+}
+
 Status condition_to_capnp(
     const QueryCondition& condition,
     capnp::Condition::Builder* condition_builder) {
-  // Serialize the clauses.
-  const std::vector<QueryCondition::Clause> clauses = condition.clauses();
-  assert(!clauses.empty());
-  auto clause_builder = condition_builder->initClauses(clauses.size());
-  for (size_t i = 0; i < clauses.size(); ++i) {
-    clause_builder[i].setFieldName(clauses[i].field_name_);
+  const tdb_unique_ptr<ASTNode>& ast = condition.ast();
+  assert(!condition.empty());
 
-    // Copy the condition value into a capnp vector of bytes.
-    const ByteVecValue value = clauses[i].condition_value_data_;
-    auto capnpValue = kj::Vector<uint8_t>();
-    capnpValue.addAll(kj::ArrayPtr<uint8_t>(
-        const_cast<uint8_t*>(reinterpret_cast<const uint8_t*>(value.data())),
-        value.size()));
+  // Validate and store the query condition AST.
+  auto ast_builder = condition_builder->initTree();
+  RETURN_NOT_OK(condition_ast_to_capnp(ast, &ast_builder));
 
-    // Store the condition value vector of bytes.
-    clause_builder[i].setValue(capnpValue.asPtr());
+  // For backwards compatability, we should also set the clauses and combination
+  // ops vectors when the current query condition.
+  if (ast->is_backwards_compatible()) {
+    if (ast->is_expr()) {
+      // We assume that the serialized values of the clauses are validated
+      // properly.
+      const std::vector<tdb_unique_ptr<ASTNode>>& clauses_vec =
+          ast->get_children();
+      auto clauses_builder = condition_builder->initClauses(clauses_vec.size());
+      for (size_t i = 0; i < clauses_vec.size(); ++i) {
+        auto clause_builder = clauses_builder[i];
+        clause_to_capnp(clauses_vec[i], &clause_builder);
+      }
 
-    const std::string op_str = query_condition_op_str(clauses[i].op_);
-    clause_builder[i].setOp(op_str);
-  }
-
-  // Serialize the combination ops.
-  const std::vector<QueryConditionCombinationOp> combination_ops =
-      condition.combination_ops();
-  if (!combination_ops.empty()) {
-    auto combination_ops_builder =
-        condition_builder->initClauseCombinationOps(combination_ops.size());
-    for (size_t i = 0; i < combination_ops.size(); ++i) {
-      const std::string op_str =
-          query_condition_combination_op_str(combination_ops[i]);
-      combination_ops_builder.set(i, op_str);
+      // Validating and storing the combination op vector.
+      if (clauses_vec.size() > 1) {
+        auto combination_ops_builder =
+            condition_builder->initClauseCombinationOps(clauses_vec.size() - 1);
+        for (size_t i = 0; i < clauses_vec.size() - 1; ++i) {
+          const std::string op_str = query_condition_combination_op_str(
+              QueryConditionCombinationOp::AND);
+          ensure_qc_combo_op_string_is_valid(op_str);
+          combination_ops_builder.set(i, op_str);
+        }
+      }
+    } else {
+      // We assume the serialized value of the clause is verified properly.
+      auto clauses_builder = condition_builder->initClauses(1);
+      auto clause_builder = clauses_builder[0];
+      clause_to_capnp(ast, &clause_builder);
     }
   }
-
   return Status::Ok();
 }
 
@@ -752,42 +875,101 @@ Status dense_reader_to_capnp(
   return Status::Ok();
 }
 
+tdb_unique_ptr<ASTNode> condition_ast_from_capnp(
+    const capnp::ASTNode::Reader& ast_reader) {
+  if (!ast_reader.getIsExpression()) {
+    // Getting and validating the field name.
+    std::string field_name = ast_reader.getFieldName();
+    ensure_qc_field_name_is_valid(field_name);
+
+    // Getting and validating the condition value.
+    auto condition_value = ast_reader.getValue();
+    const void* data =
+        static_cast<const void*>(condition_value.asBytes().begin());
+    size_t size = condition_value.size();
+    ensure_qc_condition_value_is_valid(data, size);
+
+    // Getting and validating the query condition operator.
+    QueryConditionOp op = QueryConditionOp::LT;
+    Status s = query_condition_op_enum(ast_reader.getOp(), &op);
+    if (!s.ok()) {
+      throw std::runtime_error(
+          "condition_ast_from_capnp: query_condition_op_enum failed.");
+    }
+    ensure_qc_op_is_valid(op);
+
+    return tdb_unique_ptr<ASTNode>(
+        tdb_new(ASTNodeVal, field_name, data, size, op));
+  }
+
+  // Getting and validating the query condition combination operator.
+  std::string combination_op_str = ast_reader.getCombinationOp();
+  QueryConditionCombinationOp combination_op = QueryConditionCombinationOp::AND;
+  Status s =
+      query_condition_combination_op_enum(combination_op_str, &combination_op);
+  if (!s.ok()) {
+    throw std::runtime_error(
+        "condition_ast_from_capnp: query_condition_combination_op_enum "
+        "failed.");
+  }
+  ensure_qc_combo_op_is_valid(combination_op);
+
+  // We assume that the deserialized values of the child nodes are validated
+  // properly.
+  std::vector<tdb_unique_ptr<ASTNode>> ast_nodes;
+  for (const auto child : ast_reader.getChildren()) {
+    ast_nodes.push_back(condition_ast_from_capnp(child));
+  }
+  return tdb_unique_ptr<ASTNode>(
+      tdb_new(ASTNodeExpr, std::move(ast_nodes), combination_op));
+}
+
 Status condition_from_capnp(
     const capnp::Condition::Reader& condition_reader,
     QueryCondition* const condition) {
-  // Deserialize the clauses.
-  std::vector<QueryCondition::Clause> clauses;
-  assert(condition_reader.hasClauses());
-  for (const auto clause : condition_reader.getClauses()) {
-    std::string field_name = clause.getFieldName();
+  if (condition_reader.hasClauses()) {  // coming from older API
+    // Accumulating the AST value nodes from the clause list.
+    std::vector<tdb_unique_ptr<ASTNode>> ast_nodes;
+    for (const auto clause : condition_reader.getClauses()) {
+      // Getting and validating the field name.
+      std::string field_name = clause.getFieldName();
+      ensure_qc_field_name_is_valid(field_name);
 
-    auto condition_value = clause.getValue();
+      // Getting and validating the condition value.
+      auto condition_value = clause.getValue();
+      const void* data =
+          static_cast<const void*>(condition_value.asBytes().begin());
+      size_t size = condition_value.size();
+      ensure_qc_condition_value_is_valid(data, size);
 
-    QueryConditionOp op = QueryConditionOp::LT;
-    RETURN_NOT_OK(query_condition_op_enum(clause.getOp(), &op));
+      // Getting and validating the query condition operator.
+      QueryConditionOp op = QueryConditionOp::LT;
+      Status s = query_condition_op_enum(clause.getOp(), &op);
+      if (!s.ok()) {
+        throw std::runtime_error(
+            "condition_ast_from_capnp: query_condition_op_enum failed.");
+      }
+      ensure_qc_op_is_valid(op);
 
-    clauses.emplace_back(
-        std::move(field_name),
-        condition_value.asBytes().begin(),
-        condition_value.size(),
-        op);
-  }
-  condition->set_clauses(std::move(clauses));
-
-  // Deserialize the combination ops.
-  if (condition_reader.hasClauseCombinationOps()) {
-    std::vector<QueryConditionCombinationOp> combination_ops;
-    for (const auto combination_op_str :
-         condition_reader.getClauseCombinationOps()) {
-      QueryConditionCombinationOp combination_op =
-          QueryConditionCombinationOp::AND;
-      RETURN_NOT_OK(query_condition_combination_op_enum(
-          combination_op_str, &combination_op));
-      combination_ops.emplace_back(combination_op);
+      ast_nodes.push_back(tdb_unique_ptr<ASTNode>(
+          tdb_new(ASTNodeVal, field_name, data, size, op)));
     }
-    condition->set_combination_ops(std::move(combination_ops));
-  }
 
+    // Constructing the tree from the list of AST nodes.
+    assert(ast_nodes.size() > 0);
+    if (ast_nodes.size() == 1) {
+      condition->set_ast(std::move(ast_nodes[0]));
+    } else {
+      auto tree_ptr = tdb_unique_ptr<ASTNode>(tdb_new(
+          ASTNodeExpr, std::move(ast_nodes), QueryConditionCombinationOp::AND));
+      condition->set_ast(std::move(tree_ptr));
+    }
+  } else if (condition_reader.hasTree()) {
+    // Constructing the query condition from the AST representation.
+    // We assume that the deserialized values of the AST are validated properly.
+    auto ast_reader = condition_reader.getTree();
+    condition->set_ast(condition_ast_from_capnp(ast_reader));
+  }
   return Status::Ok();
 }
 
