@@ -638,13 +638,13 @@ class StorageManager {
    * @param config The configuration parameters.
    * @return Status
    */
-  Status init(const Config* config);
+  Status init(const Config& config);
 
   /** Returns the thread pool for compute-bound tasks. */
-  ThreadPool* compute_tp();
+  ThreadPool* compute_tp() const;
 
   /** Returns the thread pool for io-bound tasks. */
-  ThreadPool* io_tp();
+  ThreadPool* io_tp() const;
 
   /**
    * If the storage manager was configured with a REST server, return the
@@ -989,6 +989,31 @@ class StorageManager {
   /** Returns the internal logger object. */
   shared_ptr<Logger> logger() const;
 
+  /**
+   * Consolidates the metadata of a group into a single file.
+   *
+   * @param group_name The name of the group whose metadata will be
+   *     consolidated.
+   * @param config Configuration parameters for the consolidation
+   *     (`nullptr` means default, which will use the config associated with
+   *      this instance).
+   * @return Status
+   */
+  Status group_metadata_consolidate(
+      const char* group_name, const Config* config);
+
+  /**
+   * Vacuums the consolidated metadata files of a group.
+   *
+   * @param group_name The name of the group whose metadata will be
+   *     vacuumed.
+   * @param config Configuration parameters for vacuuming
+   *     (`nullptr` means default, which will use the config associated with
+   *      this instance).
+   * @return Status
+   */
+  Status group_metadata_vacuum(const char* group_name, const Config* config);
+
  private:
   /* ********************************* */
   /*        PRIVATE DATATYPES          */
