@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2022 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -33,9 +33,10 @@
 #ifndef TILEDB_TILE_H
 #define TILEDB_TILE_H
 
+#include "tiledb/common/common.h"
 #include "tiledb/common/status.h"
 #include "tiledb/sm/array_schema/attribute.h"
-#include "tiledb/sm/buffer/buffer.h"
+#include "tiledb/sm/tile/filtered_buffer.h"
 
 #include <cinttypes>
 
@@ -67,6 +68,13 @@ class Tile {
       const uint64_t tile_cell_size,
       uint32_t* const chunk_size);
 
+  /**
+   * Override max_tile_chunk_size_. Used in tests only.
+   *
+   * @param max_tile_chunk_size The maximum chunk size.
+   */
+  static void set_max_tile_chunk_size(uint64_t max_tile_chunk_size);
+
   /* ********************************* */
   /*     CONSTRUCTORS & DESTRUCTORS    */
   /* ********************************* */
@@ -81,66 +89,41 @@ class Tile {
    * @param cell_size The cell size.
    * @param dim_num The number of dimensions in case the tile stores
    *      coordinates.
-   * @param Buffer The buffer to be encapsulated by the tile object. This means
-   *     that the tile will not create a new Buffer object, but operate
-   *     on the input.
-   * @param owns_buff If *true* the tile object will delete *buffer_* upon
-   *     destruction, otherwise it will not delete it.
+   * @param buffer The buffer to be encapsulated by the tile object. This means
+   *     that the tile will take ownership of the buffer.
+   * @param size The buffer size.
    */
   Tile(
       Datatype type,
       uint64_t cell_size,
       unsigned int dim_num,
-      Buffer* buffer,
-      bool owns_buff);
-
-  /**
-   * Constructor.
-   *
-   * @param format_version The format version of the internal buffer.
-   * @param type The type of the data to be stored.
-   * @param cell_size The cell size.
-   * @param dim_num The number of dimensions in case the tile stores
-   *      coordinates.
-   * @param Buffer The buffer to be encapsulated by the tile object. This means
-   *     that the tile will not create a new Buffer object, but operate
-   *     directly on the input.
-   * @param owns_buff If *true* the tile object will delete *buffer_* upon
-   *     destruction, otherwise it will not delete it.
-   */
-  Tile(
-      uint32_t format_version,
-      Datatype type,
-      uint64_t cell_size,
-      unsigned int dim_num,
-      Buffer* buffer,
-      bool owns_buff);
-
-  /**
-   * Copy constructor. This performs a deep copy (including potential memcpy of
-   * underlying buffers).
-   */
-  Tile(const Tile& tile);
+      void* buffer,
+      uint64_t size);
 
   /** Move constructor. */
   Tile(Tile&& tile);
 
-  /** Destructor. */
-  ~Tile();
-
-  /**
-   * Copy-assign operator. This performs a deep copy (including potential memcpy
-   * of underlying buffers).
-   */
-  Tile& operator=(const Tile& tile);
-
   /** Move-assign operator. */
   Tile& operator=(Tile&& tile);
 
+  DISABLE_COPY_AND_COPY_ASSIGN(Tile);
+
   /* ********************************* */
   /*                API                */
   /* ********************************* */
 
+  /** Converts the data pointer to a specific type. */
+  template <class T>
+  inline T* data_as() const {
+    return static_cast<T*>(data());
+  }
+
+  /** Gets the size, considering the data as a specific type. */
+  template <class T>
+  inline size_t size_as() const {
+    return size() / sizeof(T);
+  }
+
   /** Returns the number of cells stored in the tile. */
   uint64_t cell_num() const;
 
@@ -184,35 +167,32 @@ class Tile {
       uint64_t cell_size,
       unsigned int dim_num);
 
-  /** Advances the buffer offset. */
-  void advance_offset(uint64_t nbytes);
-
   /** Returns the internal buffer. */
-  Buffer* buffer() const;
+  inline void* data() const {
+    return data_.get();
+  }
+
+  /** Clears the internal buffer. */
+  void clear_data();
+
+  /**
+   * Allocate the internal buffer.
+   *
+   * @param size New size.
+   * @return Status.
+   */
+  Status alloc_data(uint64_t size);
 
   /** Returns the cell size. */
-  uint64_t cell_size() const;
+  inline uint64_t cell_size() const {
+    return cell_size_;
+  }
 
-  /**
-   * Returns a shallow or deep copy of this Tile.
-   *
-   * @param deep_copy If true, a deep copy is performed, including potentially
-   *    memcpying the underlying Buffer. If false, a shallow copy is performed,
-   *    which sets the clone's Buffer equal to Tile's buffer pointer.
-   * @return New Tile
+  /** Returns the number of zipped coordinates (0 if this is an attribute tile).
    */
-  Tile clone(bool deep_copy) const;
-
-  /**
-   * Sets `owns_buffer_` to `false` and thus will not destroy the buffer
-   * in the destructor.
-   */
-  void disown_buff();
-
-  bool owns_buff() const;
-
-  /** Returns the number of dimensions (0 if this is an attribute tile). */
-  unsigned int dim_num() const;
+  inline unsigned int zipped_coords_dim_num() const {
+    return zipped_coords_dim_num_;
+  }
 
   /** Checks if the tile is empty. */
   bool empty() const;
@@ -221,76 +201,43 @@ class Tile {
    * Returns the current filtered state of the tile data in the buffer. When
    * `true`, the buffer contains the filtered, on-disk format of the tile.
    */
-  bool filtered() const;
+  inline bool filtered() const {
+    return filtered_buffer_.size() > 0;
+  }
 
   /**
    * Returns the buffer that contains the filtered, on-disk format.
    */
-  Buffer* filtered_buffer();
+  inline FilteredBuffer& filtered_buffer() {
+    return filtered_buffer_;
+  }
 
   /** Gets the format version number of the data in this Tile. */
-  uint32_t format_version() const;
-
-  /** Checks if the tile is full. */
-  bool full() const;
-
-  /** The current offset in the tile. */
-  uint64_t offset() const;
-
-  /**
-   * Returns the pre-filtered size of the tile data in the buffer.
-   *
-   * On writes, the pre-filtered size is the uncompressed size.
-   *
-   * On reads, the pre-filtered size is the persisted (compressed) size.
-   */
-  uint64_t pre_filtered_size() const;
-
-  /** Reads from the tile into the input buffer *nbytes*. */
-  Status read(void* buffer, uint64_t nbytes);
+  inline uint32_t format_version() const {
+    return format_version_;
+  }
 
   /**
    * Reads from the tile at the given offset into the input
    * buffer of size nbytes. Does not mutate the internal offset.
    * Thread-safe among readers.
    */
-  Status read(void* buffer, uint64_t nbytes, uint64_t offset) const;
-
-  /** Resets the size and offset of the tile. */
-  void reset();
-
-  /** Resets the tile offset. */
-  void reset_offset();
-
-  /** Resets the tile size. */
-  void reset_size();
-
-  /** Sets the tile offset. */
-  void set_offset(uint64_t offset);
-
-  /** Sets the pre-filtered size value to the given value. */
-  void set_pre_filtered_size(uint64_t pre_filtered_size);
+  Status read(void* buffer, uint64_t offset, uint64_t nbytes) const;
 
   /** Returns the tile size. */
-  uint64_t size() const;
+  inline uint64_t size() const {
+    return (data_ == nullptr) ? 0 : size_;
+  }
 
-  /** Returns *true* if the tile stores coordinates. */
-  bool stores_coords() const;
+  /** Returns *true* if the tile stores zipped coordinates. */
+  inline bool stores_coords() const {
+    return zipped_coords_dim_num_ > 0;
+  }
 
   /** Returns the tile data type. */
-  Datatype type() const;
-
-  /** Writes as much data as possibly can be read from the input buffer. */
-  Status write(ConstBuffer* buf);
-
-  /**
-   * Writes exactly `nbytes` from the input buffer to the local buffer.
-   * The local buffer can be potentially expanded to fit these bytes.
-   */
-  Status write(ConstBuffer* buf, uint64_t nbytes);
-
-  /** Writes `nbytes` from `data` to the tile. */
-  Status write(const void* data, uint64_t nbytes);
+  inline Datatype type() const {
+    return type_;
+  }
 
   /**
    * Writes `nbytes` from `data` to the tile at `offset`.
@@ -306,35 +253,37 @@ class Tile {
    */
   Status zip_coordinates();
 
- private:
+  /** Swaps the contents (all field values) of this tile with the given tile. */
+  void swap(Tile& tile);
+
+ protected:
   /* ********************************* */
-  /*         PRIVATE ATTRIBUTES        */
+  /*        PROTECTED ATTRIBUTES       */
   /* ********************************* */
 
-  /** The buffer backing the tile data. */
-  Buffer* buffer_;
+  /**
+   * The buffer backing the tile data.
+   *
+   * TODO: Convert to regular allocations once tdb_realloc is not used for var
+   * size data anymore and remove custom deleter.
+   */
+  std::unique_ptr<char, void (*)(void*)> data_;
+
+  /** Size of the data. */
+  uint64_t size_;
 
   /** The cell size. */
   uint64_t cell_size_;
 
   /**
-   * The number of dimensions, in case the tile stores coordinates. It is 0
-   * in case the tile stores attributes.
+   * The number of dimensions, in case the tile stores zipped coordinates.
+   * It is 0 in case the tile stores attributes or other type of dimensions
    */
-  unsigned int dim_num_;
+  unsigned int zipped_coords_dim_num_;
 
   /** The format version of the data in this tile. */
   uint32_t format_version_;
 
-  /**
-   * If *true* the tile object will free *buffer_* upon destruction, otherwise
-   * it will not delete it.
-   */
-  bool owns_buffer_;
-
-  /** The size in bytes of the tile data before it has been filtered. */
-  uint64_t pre_filtered_size_;
-
   /** The tile data type. */
   Datatype type_;
 
@@ -344,14 +293,13 @@ class Tile {
    * pipeline. Note that this buffer is _only_ accessed in the filtered()
    * public API, all other public API routines operate on 'buffer_'.
    */
-  Buffer filtered_buffer_;
+  FilteredBuffer filtered_buffer_;
 
-  /* ********************************* */
-  /*          PRIVATE METHODS          */
-  /* ********************************* */
-
-  /** Swaps the contents (all field values) of this tile with the given tile. */
-  void swap(Tile& tile);
+  /**
+   * Static variable to store constants::max_tile_chunk_size. This will be used
+   * to override the value in tests.
+   */
+  static uint64_t max_tile_chunk_size_;
 };
 
 }  // namespace sm
